@@ -2,23 +2,40 @@
 """
 import os
 import subprocess
+import glob
 from thornpy.signal import step_function, low_pass
 from numpy import linspace, argmax, array
 from ..postprocess.xml import get_results
 from .utilities import build, add_splines_to_acf, add_splines_to_adm
+from ..postprocess import launch_ppt
+from .string import DrillString
+from .event import DrillEvent
+from .solver_settings import DrillSolverSettings
 
 class DrillSim(): #pylint: disable=too-many-instance-attributes
-    """Contains data defining the files that make up an Adams Drill input deck.
-    
+    """Contains data defining the files that make up an Adams Drill input deck.    
+
+    Parameters
+    ----------
+    string : DrillString
+        :class:`DrillString` object representing the string to be used in the simulation
+    event : DrillEvent
+        :class:`DrillEvent` object representing the string to be used in the simulation
+    solver_settings : DrillSolverSettings
+        :class:`DrillSolverSettings` object representing the string to be used in the simulation
+    directory : str
+        Path to the directory in which to put the simulation files
+    analysis_name : str
+        Name of the analysis.  Used for all file prefixes
 
     Attributes
     ----------
     string : DrillString
-        DrillString object representing the string to be used in the simulation
+        :class:`DrillString` object representing the string to be used in the simulation
     event : DrillEvent
-        DrillEvent object representing the string to be used in the simulation
+        :class:`DrillEvent` object representing the string to be used in the simulation
     solver_settings : DrillSolverSettings
-        DrillSolverSettings object representing the string to be used in the simulation
+        :class:`DrillSolverSettings` object representing the string to be used in the simulation
     directory : str
         Path to the directory in which to put the simulation files
     analysis_name : str
@@ -33,10 +50,16 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         Filename of the analysis' cmd file
     res_filename : str
         Filename of the analysis' res file
+    results : dict
+        Simulation results
+    results_units : dict
+        Units of simulation results
     built : bool
         Indicates whether the input deck (adm, acf, and cmd files) has been built yet for this DrillSim
     RAMP_TIME : dict
         Class attribute containing standard ramp times for rpm, gpm, wob, and rop
+    CUTOFF_FREQ : dict
+        Class attribute containing default cutoff frequencies for smoothing pason inputs.
         
     Example
     -------
@@ -54,19 +77,6 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
 
     def __init__(self, string, event, solver_settings, directory, analysis_name, write_TO_files=True): #pylint: disable=too-many-arguments
         """Sets instance attributes and writes the string, event, and solver settings files to `directory`.
-
-        Parameters
-        ----------
-        string : DrillString
-            :obj:`DrillString` object representing the string to be used in the simulation
-        event : DrillEvent
-            :obj:`DrillEvent` object representing the string to be used in the simulation
-        solver_settings : DrillSolverSettings
-            :obj:`DrillSolverSettings` object representing the string to be used in the simulation
-        directory : str
-            Path to the directory in which to put the simulation files
-        analysis_name : str
-            Name of the analysis.  Used for all file prefixes
         
         """
         self.string = string
@@ -80,7 +90,8 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         self.cmd_filename = ''
         self.res_filename = ''  
         self.msg_filename = ''      
-        self.results = None
+        self.results = None     
+        self.results_units = None
         
         # Write the TO files to the working directory
         if write_TO_files:
@@ -92,6 +103,69 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         # Flags
         self.built = False
 
+    @classmethod
+    def read_from_directory(cls, directory):
+        """Returns a :class:`DrillSim` object from an existing directory that was previously created by a :class:`DrillSim` object.
+        
+        Note
+        ----
+        This method does not read the `pason_inputs` attribute
+
+        Parameters
+        ----------
+        directory : str
+            Path to a directory previously created by a :class:`DrillSim` object.
+        
+        Returns
+        -------
+        DrillSim
+            :class:`DrillSim` object created from an existing directory that was previously created by a :class:`DrillSim` object.
+
+        """
+        # If `directory` is not a directory, raise an error
+        if not os.path.isdir(directory):
+            raise OSError(f'No directory found with name {directory}!')
+
+        # Get string, event, ssf filenames
+        string_filename = glob.glob(os.path.join(directory, '*.' + DrillString._EXT))[0]
+        event_filename  = glob.glob(os.path.join(directory, '*.' + DrillEvent._EXT))[0]
+        solver_settings_filename = glob.glob(os.path.join(directory, '*.' + DrillSolverSettings._EXT))[0]
+
+        # Create string, event, ssf objects
+        string = DrillString.read_from_file(string_filename)
+        event = DrillEvent.read_from_file(event_filename)
+        solver_settings = DrillSolverSettings.read_from_file(solver_settings_filename)
+
+        # Get analysis name
+        analysis_name = string.parameters['ModelName']
+
+        # Create the DrillSim object
+        drill_sim = cls(string, event, solver_settings, directory, analysis_name, write_TO_files=False)
+
+        # Define file names
+        drill_sim.string_filename = os.path.split(string_filename)[1]   
+        
+        adm_files = glob.glob(os.path.join(directory, '*.adm'))
+        drill_sim.adm_filename = os.path.split(adm_files[0])[1] if adm_files else ''
+
+        acf_files = glob.glob(os.path.join(directory, '*.acf'))
+        drill_sim.acf_filename = os.path.split(acf_files[0])[1] if acf_files else ''
+
+        cmd_files = glob.glob(os.path.join(directory, '*.cmd'))
+        drill_sim.cmd_filename = os.path.split(cmd_files[0])[1] if cmd_files else ''
+
+        res_files = glob.glob(os.path.join(directory, '*.res'))
+        drill_sim.res_filename = os.path.split(res_files[0])[1] if res_files else ''
+
+        msg_files = glob.glob(os.path.join(directory, '*.msg'))
+        drill_sim.msg_filename = os.path.split(msg_files[0])[1] if msg_files else ''
+
+        # Set the `built` flag
+        if drill_sim.acf_filename and drill_sim.adm_filename:            
+            drill_sim.built = True
+
+        return drill_sim
+
     def get_pason_inputs(self, pason_data, t_min=None, t_max=None, sig_types=['wob', 'rpm', 'gpm', 'rop'], show_plots=True):
         """Extracts the wob, rpm, gpm, and/or rop from a :class:`PasonData` object.  If this is run before :meth:`build` the pason data will be added to the adm file as model input splines.
         
@@ -206,8 +280,33 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
             Maximum time for which to get results (the default is None, which gets results up to the last time step)
         
         """
-        self.results = get_results(self.res_filename, reqs_to_read, t_min, t_max)
-    
+        res_file = os.path.join(self.directory, self.res_filename)
+        self.results, self.results_units = get_results(res_file, reqs_to_read, t_min, t_max, return_units=True)
+
+    def launch_ppt(self, wait=False):
+        """Launches the postprocessor and loads :attr:`res_filename`
+        
+        Parameters
+        ----------
+        wait : bool, optional
+            If `True`, code execution will freeze until the postprocessor is closed. (the default is False)
+        
+        Raises
+        ------
+        FileNotFoundError
+            Raised if no results file exists in :attr:`directory`.
+        
+        """
+        # Check for results file
+        if not self.res_filename:
+            res_files = glob.glob(os.path.join(self.directory, '*.res'))
+            if not res_files:
+                raise FileNotFoundError('No results file found!  You must run the simultion using DrillSim.solve before you can launch the postprocessor to view the results.')
+            self.res_filename = os.path.split(res_files[0])[1]
+        
+        # Launch the postprocessor
+        launch_ppt(os.path.join(self.directory, self.res_filename), wait=wait)
+
     @classmethod
     def clean_pason(cls, pason_data, sig_type, t_min=None, t_max=None, show_plot=True, ramp_time=None, cutoff_freq=None):
         """Returns a cleaned up pason signal to be used as input to an Adams Drill model.
@@ -242,7 +341,7 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
             i_min = argmax(array(time)>=t_min)
             i_max = argmax(array(time)>=t_max)
             signal = signal[i_min:i_max]
-            time = time[i_min:i_max]            
+            time = time[i_min:i_max] - time[i_min]
             return signal, time   
 
         if sig_type in ['rpm', 'gpm']:
@@ -272,7 +371,7 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
             pason_data.plt.title(sig_type.upper())
             pason_data.plt.show()
 
-        return signal, time
+        return list(signal), list(time)
     
     @classmethod
     def get_filtered_signal(cls, pason_data, sig_type, cutoff_freq=None):
