@@ -1,15 +1,17 @@
 """A module that contains the :class:`DrillString` class
 """
 import os
-import shutil
+import copy
 import re
+import shutil
 from . import TMPLT_ENV
 from .tool import DrillTool
 from .utilities import read_TO_file, get_cdb_location, get_cdb_path, get_full_path, TO_LENGTH_PARAM, isabs
+from .constants import DATABASE_INFO
 
 class DrillString():
     """
-    Creates an object with all data necessary to write a drill string.      
+    An object with all data necessary to write a drill string.      
     
     Parameters in the string file can be specified when the DrillString is instanced using kwargs or they can be set later using: 
         
@@ -29,7 +31,6 @@ class DrillString():
         Describes the top drive.  The keys are 'DrillTool', 'Type', 'Name', and 'Property_File'.
         
     """
-
     _SCALAR_PARAMETERS = [
         'Units',
         'ModelName',
@@ -61,12 +62,12 @@ class DrillString():
         'Hole_Color': 'LtGray'
     }
 
-    _ARRAY_PARAMETERS = [
+    _TABLE_PARAMETERS = [
         'Distance_from_Bit'
     ]
 
-    _DEFAULT_PARAMETER_ARRAYS = {
-        'Distance_from_Bit': []
+    _DEFAULT_PARAMETER_TABLES = {
+        'Distance_from_Bit': ()
     }
 
     _CDB_TABLE = 'drill_strings.tbl'
@@ -88,7 +89,6 @@ class DrillString():
             Filename of an event file.
         
         """
-
         self.parameters = kwargs
         self.parameters['ModelName'] = string_name
         self.parameters['OutputName'] = string_name
@@ -110,7 +110,7 @@ class DrillString():
         
         Parameters
         ----------
-        tool : DrillTool
+        drill_tool : DrillTool
             :class:`DrillTool` object representing the tool to be added        
         joints : int
             Number of Joints. Note that this only applies for certain tool types. (default is 1)
@@ -120,6 +120,11 @@ class DrillString():
             If an integer is given the tool will be inserted into the string at that point. (default is None which means the tool will be appended to the end)
         color : str
             The color used to render the tool in an Adams Drill animation. (default is 'Default')
+        group_name : str
+            This argument is required if the tool being added is a tool that comes in multiple joints (e.g. drill_pipe).  NOTE: Tools that come in multiple joints are listed in :attr:`DrillTool._MULTI_JOINT_TOOLS`.
+        eqivalent : bool
+            Set this to `True` if the tool being added is drill_pipe and you want to use the equivalent upper string formulation.
+            
         """
         # Check that the group_name argument is givn
         if drill_tool.tool_type.lower() in self._MULTI_JOINT_TOOLS and group_name is None:
@@ -208,7 +213,7 @@ class DrillString():
             raise DrillStringError(f'There is no {tool_type} in this string!')        
     
     def get_tool(self, tool_type, index=0):
-        """Returns a DrillTool object of type :arg:`tool_type` in the :class:`DrillString` object's tools list.  
+        """Returns a DrillTool object of type `tool_type` in the :class:`DrillString` object's tools list.  
         
         Parameters
         ----------
@@ -249,7 +254,7 @@ class DrillString():
         return tools_found[index]
         
     def tool_renamed(self, renamed_tool):
-        """Updates the 'name' and 'Property_File' entries in the :attr:`tools` to match :arg:`renamed_tool`.name.        
+        """Updates the 'name' and 'Property_File' entries in the :attr:`tools` to match `renamed_tool`.name.        
 
         Parameters
         ----------
@@ -323,9 +328,9 @@ class DrillString():
         cdb : str
             Name of the cdb in which to write the file.  This argument is overridden by `directory`. (default is None which means the string will be written to the directory in `directory` and will use the filename in `filename`.)  
         publish : bool
-            If true, writes all the supporting files to the same cdb. (default is False.)
+            If True, writes all the supporting files except the event file to the same cdb or directory. (default is False.)
         publish_event : bool
-
+            If True, writes the event file to the same cdb or directory. (default is False.)
         
         Raises
         ------
@@ -517,13 +522,13 @@ class DrillString():
         # Applies normal parameter defaults
         for scalar_parameter, value in self._DEFAULT_PARAMETER_SCALARS.items():
             if scalar_parameter not in self.parameters:
-                self.parameters[scalar_parameter] = value
+                self.parameters[scalar_parameter] = copy.copy(value)
 
         # Applies defaults to all ramp parameters
-        for array_parameter, array in self._DEFAULT_PARAMETER_ARRAYS.items():
-            self.parameters[array_parameter] = {}
-            self.parameters[array_parameter] = array
-            self.parameters['_' + array_parameter] = zip(*self.parameters[array_parameter])
+        for table_parameter, table in self._DEFAULT_PARAMETER_TABLES.items():
+            self.parameters[table_parameter] = {}
+            self.parameters[table_parameter] = [list(tup) for tup in table]
+            self.parameters['_' + table_parameter] = zip(*self.parameters[table_parameter])
     
     def _copy_hole_file(self, directory=None, cdb=None):
         """Copys the hole file to a new locations.  Must give directory OR cdb.
@@ -534,17 +539,18 @@ class DrillString():
             Directory to put the copied hole file (default is None)
         cdb : str
             Name of cdb to put the copied hole file (default is None)
+
         """
         # Check that directory or cdb was given.
         if cdb is None and directory is None:
             raise ValueError('Either directory or cdb is required!')
 
         # Get the hole name from the current file
-        hole_name = os.path.split(self.parameters['Hole_Property_File'])[-1].replace('.' + DrillTool._DATABASE_INFO['hole']['extension'], '') + '.' + DrillTool._DATABASE_INFO['hole']['extension']
+        hole_name = os.path.split(self.parameters['Hole_Property_File'])[-1].replace('.' + DATABASE_INFO['hole']['extension'], '') + '.' + DATABASE_INFO['hole']['extension']
 
         # Set the new filename
         if cdb is not None:
-            new_filename = os.path.join(get_cdb_location(cdb), DrillTool._DATABASE_INFO['hole']['table'], hole_name)
+            new_filename = os.path.join(get_cdb_location(cdb), DATABASE_INFO['hole']['table'], hole_name)
         else:
             new_filename = os.path.join(directory, hole_name)
 
@@ -567,17 +573,18 @@ class DrillString():
         Note
         ----
         Either directory or cdb must be given.
+        
         """
         # Check that directory or cdb was given.
         if cdb is None and directory is None:
             raise ValueError('Either directory or cdb is required!')
 
         # Get the event name from the current file
-        event_name = os.path.split(self.parameters['Event_Property_File'])[-1].replace('.' + DrillTool._DATABASE_INFO['event']['extension'], '') + '.' + DrillTool._DATABASE_INFO['event']['extension']
+        event_name = os.path.split(self.parameters['Event_Property_File'])[-1].replace('.' + DATABASE_INFO['event']['extension'], '') + '.' + DATABASE_INFO['event']['extension']
 
         # Set the new filename
         if cdb is not None:
-            new_filename = os.path.join(get_cdb_location(cdb), DrillTool._DATABASE_INFO['event']['table'], event_name)
+            new_filename = os.path.join(get_cdb_location(cdb), DATABASE_INFO['event']['table'], event_name)
         else:
             new_filename = os.path.join(directory, event_name)
 
@@ -660,7 +667,7 @@ class DrillString():
         # Read the tiem orbit data into a dictionary
         tiem_orbit_data = read_TO_file(tiem_orbit_file)
         
-        for param in self._SCALAR_PARAMETERS + self._ARRAY_PARAMETERS + self._FILENAME_PARAMETERS:
+        for param in self._SCALAR_PARAMETERS + self._TABLE_PARAMETERS + self._FILENAME_PARAMETERS:
             # For each string parameter initialize a found flag
             param_value = None
         
