@@ -45,12 +45,12 @@ class Test_DrillString(unittest.TestCase):
         
         # Create a DrillTool object representing a top drive
         self.top_drive = adripy.DrillTool(TEST_TOP_DRIVE_FILE)
-
+        
     def test_add_tool(self):
         """Test the `DrillString.add_tool()` method.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool object to the DrillString object
         drill_string.add_tool(self.stabilizer)
@@ -73,7 +73,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests if the string file is written correctly when the string has a collar in it.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -96,7 +96,7 @@ class Test_DrillString(unittest.TestCase):
         """Test the `DrillString.write_to_file()` method.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -118,7 +118,7 @@ class Test_DrillString(unittest.TestCase):
         """Test the `DrillString.write_to_file()` method.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add DFBs to the drill string
         drill_string.parameters['Distance_from_Bit'] = [100, 200]
@@ -138,12 +138,33 @@ class Test_DrillString(unittest.TestCase):
         failures = check_file_contents(expected_string_filename, EXPECTED_STRING_WRITE_TEXT_WITH_DFB)
 
         self.assertListEqual(failures, [])
+    
+    def test_publish_string_with_spaces_in_hole_path(self):
+        """Tests if publish works when the hole path has spaces        
+        """
+        # Create a DrillString object
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE_WITH_SPACES, TEST_EVENT_FILE)
+
+        # Add the DrillTool objects to the DrillString object
+        drill_string.add_tool(self.pdc_bit, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='yes')
+        drill_string.add_tool(self.collar, measure='yes', joints=TEST_NUMBER_OF_COLLARS, group_name=TEST_COLLAR_GROUPNAME)
+        drill_string.add_tool(self.drill_pipe, joints=TEST_NUMBER_OF_DRILLPIPES, group_name=TEST_DRILLPIPE_GROUPNAME)
+        drill_string.add_tool(self.eus, joints=TEST_NUMBER_OF_EUSPIPES, group_name=TEST_EUS_GROUPNAME, equivalent=True)
+        drill_string.add_tool(self.top_drive)
+
+        # Write drill string to file
+        try:
+            drill_string.write_to_file(cdb=TEST_NEW_DATABASE_NAME, publish=True)
+            pass
+        except FileNotFoundError:
+            self.fail()
 
     def test_publish_string_to_new_database(self):
         """Test the `DrillString.write_to_file()` method with publish=True
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -170,7 +191,7 @@ class Test_DrillString(unittest.TestCase):
             This test intentionally uses a string that uses the same tool twice.            
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -197,7 +218,7 @@ class Test_DrillString(unittest.TestCase):
         This test intentionally uses a string that uses the same tool twice.            
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -216,6 +237,18 @@ class Test_DrillString(unittest.TestCase):
 
         self.assertListEqual(actual_files, expected_files)    
 
+    def test_read_string_from_file_with_spaces_in_hole_ref(self):
+        """Tests that the parameters in the string are correct after a string is read from a file.
+        """
+        # Read new parameters into the drill string object from a file
+        string_file = os.path.join(f'<{TEST_DATABASE_NAME}>', 'drill_strings.tbl', TEST_EXISTING_STRING_NAME_WITH_SPACES_IN_HOLE_REF + '.str')
+        drill_string_from_file = adripy.DrillString.read_from_file(string_file)
+        
+        # Remove some keys from the parameters dictionary
+        hole_filename = drill_string_from_file.parameters['Hole_Property_File']
+        
+        self.assertEqual(hole_filename, TEST_EXISTING_HOLE_FILE_WITH_SPACES)
+
     def test_read_string_from_file_parameters(self):
         """Tests that the parameters in the string are correct after a string is read from a file.
         """
@@ -434,7 +467,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests that the string is read correctly when the file uses relative references.
         """
         
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -468,7 +501,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests that DrillString.get_tool() returns the correct value.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Define a new stabilizer object
         different_stabilizer  = adripy.DrillTool(TEST_STABILIZER_FILE)
@@ -489,7 +522,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests that DrillString.get_tool() returns the correct value.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
         
         # Define a new stabilizer object
         different_stabilizer  = adripy.DrillTool(TEST_STABILIZER_FILE)
@@ -510,7 +543,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests that DrillString.get_tool() returns the correct value.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
         
         # Define a new stabilizer object
         different_stabilizer  = adripy.DrillTool(TEST_STABILIZER_FILE)
@@ -531,7 +564,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests that DrillString.get_tool() returns the correct value.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
         
         # Define a new stabilizer object
         different_stabilizer  = adripy.DrillTool(TEST_STABILIZER_FILE)
@@ -554,7 +587,7 @@ class Test_DrillString(unittest.TestCase):
         """
         expected_joints = 100
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -576,7 +609,7 @@ class Test_DrillString(unittest.TestCase):
         """
         expected_joints = 100
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -596,7 +629,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests that DrillString.get_bha_length() returns the correct length.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -615,7 +648,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests that changing the name of a :class:`DrillTool` object causes the 'Name' and 'Property_File' values in :attr:`DrillString.tools` to change as well.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
@@ -644,7 +677,7 @@ class Test_DrillString(unittest.TestCase):
         """Tests that changing the name of a :class:`DrillTool` object causes the 'Name' and 'Property_File' values in :attr:`DrillString.tools` to change as well.
         """
         # Create a DrillString object
-        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
 
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
